#! /usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const program = require("commander");
const fs_1 = require("fs");
const globby_1 = require("globby");
const path_1 = require("path");
const util_1 = require("./util");
program
    .version('0.0.1')
    .option('-p, --project <file>', 'path to tsconfig.json')
    .option('-s, --src <path>', 'source root path')
    .option('-o, --out <path>', 'output root path');
program.on('--help', () => {
    console.log(`
  $ tscpath -p tsconfig.json
`);
});
program.parse(process.argv);
const { project, src, out } = program;
if (!project) {
    throw new Error('--project must be specified');
}
if (!src) {
    throw new Error('--src must be specified');
}
const configFile = path_1.resolve(process.cwd(), project);
console.log(`tsconfig.json: ${configFile}`);
const srcRoot = path_1.resolve(src);
console.log(`src: ${srcRoot}`);
const outRoot = out && path_1.resolve(out);
console.log(`out: ${outRoot}`);
const { baseUrl, outDir, paths } = util_1.loadConfig(configFile);
if (!baseUrl) {
    throw new Error('compilerOptions.baseUrl is not set');
}
if (!paths) {
    throw new Error('compilerOptions.paths is not set');
}
if (!outDir) {
    throw new Error('compilerOptions.outDir is not set');
}
console.log(`baseUrl: ${baseUrl}`);
console.log(`outDir: ${outDir}`);
console.log(`paths: ${JSON.stringify(paths, null, 2)}`);
const configDir = path_1.dirname(configFile);
console.log(`configDir: ${configDir}`);
const basePath = path_1.resolve(configDir, baseUrl);
console.log(`basePath: ${basePath}`);
const outPath = outRoot || path_1.resolve(basePath, outDir);
console.log(`outPath: ${outPath}`);
const aliases = Object.keys(paths)
    .map((alias) => ({
    prefix: alias.replace(/\*$/, ''),
    paths: paths[alias].map((p) => p.replace(/\*$/, '')
        .replace('src', '.')
        .replace('ts', 'js')),
}))
    .filter(({ prefix }) => prefix);
console.log(`aliases: ${JSON.stringify(aliases, null, 2)}`);
const requireRegex = /(?:import|require)\(['"]([^'"]*)['"]\)/g;
const importRegex = /(?:import|from) ['"]([^'"]*)['"]/g;
const replaceImportStatement = (orig, matched, aliasPath) => {
    const index = orig.indexOf(matched);
    const isAlias = orig.indexOf('/' + matched) === -1 ? true : false;
    if (index > -1 && isAlias) {
        const path = orig.substring(0, index) +
            aliasPath +
            orig.substring(index + matched.length);
        return path;
    }
    return orig;
};
const replaceAlias = (text) => {
    for (const alias of aliases) {
        text = text
            .replace(requireRegex, (orig) => replaceImportStatement(orig, alias.prefix, alias.paths[0]))
            .replace(importRegex, (orig) => replaceImportStatement(orig, alias.prefix, alias.paths[0]));
    }
    return text;
};
const files = globby_1.sync(`${outPath}/**/*.{js,jsx,ts,tsx}`, {
    dot: true,
    noDir: true,
}).map((x) => path_1.resolve(x));
const flen = files.length;
for (let i = 0; i < flen; i += 1) {
    const file = files[i];
    const text = fs_1.readFileSync(file, 'utf8');
    const newText = replaceAlias(text);
    if (text !== newText) {
        fs_1.writeFileSync(file, newText, 'utf8');
    }
}
//# sourceMappingURL=index_.js.map